
#include <Python.h>
#include <tcl.h>

/* type definitions */
typedef struct {
    Tcl_Event tcl_event;
    PyObject *callable;
} python_call_event;

/* internal functions */
static int call_callable(PyObject *py_callback);
static int callable_arg(PyObject *func, void *func_p);
static int handle_event(Tcl_Event *evPtr, int flags);
static int widget_tcl_interpreter_arg(PyObject *w, void *ti);

/* functions implementing module methods */
static PyObject *queue_call(PyObject *self, PyObject *args);

/* module method definitions */
static PyMethodDef DeadlocktestMethods[] = {
    { "queue_call", queue_call, METH_VARARGS, NULL },
    { NULL, NULL, 0, NULL}
};

/*
 * module method implementation:
 * Queues a call to the specified Python Callable to be performed by the Tcl
 * event loop of the Tcl interpreter associated with the current thread.
 */
static PyObject *
queue_call(PyObject *self, PyObject *args)
{
    PyObject *callback;
    python_call_event *event;

    if (!PyArg_ParseTuple(args, "O&", callable_arg, &callback))
        return NULL;

    /* Create and initialize a python_call_event */
    event = (python_call_event *) ckalloc(sizeof(python_call_event));
    if (event == NULL) {
        PyErr_SetString(PyExc_MemoryError,
                        "cannot allocate memory for a Tcl event");
        return NULL;
    }
    event->tcl_event.proc = handle_event;
    event->callable = callback;

    /* Queue the event for THIS THREAD'S Tcl interpreter */
    Tcl_QueueEvent(&(event->tcl_event), TCL_QUEUE_HEAD);

    /*
     * We need to own a reference to the callback; it will be released once
     * the callback has been called by handle_event
     */
    Py_INCREF(callback);

    Py_RETURN_NONE;
}

/*
 * Handles a python_call_event by calling the associated callable
 */
static int handle_event(Tcl_Event *event, int flags)
{
    python_call_event *call_event = (python_call_event *) event;

    call_callable(call_event->callable);
    Py_DECREF(call_event->callable);

    return 1;
}

static int call_callable(PyObject *callable)
{
    /*
     * When this function is called via tkinter, the current thread might not
     * (probably doesn't) hold the GIL
     */
    PyGILState_STATE gstate = PyGILState_Ensure();

    PyObject *result = PyObject_CallFunction(callable, NULL);

    Py_XDECREF(result);
    /* need to PyErr_Clear() if result is NULL ? */
    PyGILState_Release(gstate);

    return ((result == NULL) ? 0 : 1);
}

/*
 * A Converter used with PyArg_ParseTuple() O& format to verify that an
 * argument is Callable.  If so, the argument is returned as-is; otherwise a
 * conversion error is reported.
 */
static int callable_arg(PyObject *func, void *func_p)
{
    if (!PyCallable_Check(func)) {
        PyErr_SetString(PyExc_TypeError, "argument is not a function");
        return 0;
    } else {
        *(PyObject **) func_p = func;
        return 1;
    }
}

/*
 * A Converter used with PyArg_ParseTuple() O& format to convert a tk widget
 * to a pointer to the Tcl interpreter to which that widget belongs.
 */
static int widget_tcl_interpreter_arg(PyObject *w, void *ti)
{
    /*
     * GCC complains if the method name is passed to PyObject_CallMethod as
     * a string literal
     */
    char methodname[] = "interpaddr"; 
    PyObject *interpaddrobj = PyObject_CallMethod(w, methodname, NULL);

    if (interpaddrobj != NULL) {
        long interpaddr = PyInt_AsLong(interpaddrobj);
        Py_DECREF(interpaddrobj);

        if (PyErr_Occurred() == NULL) {
            *((Tcl_Interp **) ti) = (Tcl_Interp *) interpaddr;
            return 1;
        } /* else not a number */
    } /* else no interpreter */

    PyErr_SetString(PyExc_TypeError, "not a Tk widget");

    return 0;
}

/* module initialization function */
#ifdef PYTHON2

PyMODINIT_FUNC
initdeadlocktest(void)
{
    Py_InitModule("deadlocktest", DeadlocktestMethods);
}

#else

static struct PyModuleDef deadlocktestmodule = {
    PyModuleDef_HEAD_INIT,
    "deadlocktest",
    NULL,
    -1,
    DeadlocktestMethods
};

PyMODINIT_FUNC
PyInit_deadlocktest(void)
{
    return PyModule_Create(&deadlocktestmodule);
}

#endif

