#
# Provides configuration options and limited searches for the TCL and TK libraries and headers.
# 
#
AC_DEFUN([SJ_LIB_TCLTK], [

  dnl Declare the --with-tcl option and the overriding TCL_LIBS and TCL_CPPFLAGS variables
  AC_ARG_WITH([tcl], AS_HELP_STRING([--with-tcl=<path>],
      [Specifies the base install location of the TCL package, if it is not in the default search path.  Ignored if TCL_LIBS or TCL_CPPFLAGS is set.]),
    [AS_IF([test -d "$withval"], [tcl_base="$withval"],
        [AC_MSG_ERROR([[Specified TCL directory (${withval}) does not exist]])])
    ], [])

  AC_ARG_VAR([TCL_LIBS], 
      [If set and not null, explicitly specifies the linker flags for linking with the TCL libraries.  Where possible, it is better to use --with-tcl or to rely on the default search path.  If you need to use this variable then you may also need to use TCL_CPPFLAGS.])

  AC_ARG_VAR([TCL_CPPFLAGS], 
      [If set and not null, explicitly specifies the preprocessor flags for compilation of sources dependant on the TCL library.  Where possible, it is better to use --with-tcl or to rely on the default search path.  If you need to use this variable then you probably also need to use TCL_LIBS.])

  dnl Declare the --with-tk option and the overriding TK_LIBS and TK_CPPFLAGS variables
  AC_ARG_WITH([tk], AS_HELP_STRING([--with-tk=<path>],
      [Specifies the base install location of the Tk package, if it is not in the default search path.  Ignored if TK_LIBS or TK_CPPFLAGS is set.]),
    [AS_IF([test -d "$withval"], [tk_base="$withval"],
        [AC_MSG_ERROR([[Specified Tk directory (${withval}) does not exist]])])
    ], [])

  AC_ARG_VAR([TK_LIBS],
      [If set and not null, explicitly specifies the linker flags for linking with the Tk libraries.  Where possible, it is better to use --with-tk or to rely on the default search path.  If you need to use this variable then you may also need to use TK_CPPFLAGS.])

  AC_ARG_VAR([TK_CPPFLAGS],
      [If set and not null, explicitly specifies the preprocessor flags for compilation of sources dependant on the Tk library.  Where possible, it is better to use --with-tk or to rely on the default search path.  If you need to use this variable then you probably also need to use TK_LIBS.])


  # Save the current values of the LIBS abd CPPFLAGS variables
  tcltk_saved_LIBS=${LIBS}
  tcltk_saved_CPPFLAGS=${CPPFLAGS}

  #
  # Find / confirm the location of the tcl and Tk libraries and headers
  #

  # Defer to user-specified TCL_LIBS and/or TCL_CPPFLAGS if they are provided
  AS_IF([test "x${TCL_LIBS}${TCL_CPPFLAGS}" = x], [
    AS_IF([test "x$tcl_base" = x], [
      # --with-tcl was not given, so just check the default search path
      AC_CHECK_LIB([tcl], [Tcl_Main], [TCL_LIBS="-ltcl" LIBS="-ltcl ${LIBS}"],
        [AC_MSG_ERROR([TCL library is not found.  Try specifing its location via --with-tcl])])
    ], [
      # --with-tcl was given, so attempt to find the headers and libs under the specified base
      AS_IF([test -r "${tcl_base}/include/tcl.h"], [
        TCL_CPPFLAGS="-I${tcl_base}/include"
        CPPFLAGS="${TCL_CPPFLAGS} ${CPPFLAGS}"
      ], [AC_MSG_WARN([${tcl_base}/include/tcl.h does not exist or is not readable.])])

      AC_MSG_CHECKING([for TCL libs in user-specified base directory ${tcl_base}])

      AC_LANG_PUSH([C])
      AS_UNSET([TCL_LIBS])
      # check lib64/ first to find 64-bit libs on systems that put 32-bit libs in lib/
      for libdir in lib64 lib lib32; do
        trydir="${tcl_base}/${libdir}"
        AS_IF([test -d "${trydir}"], [
          LIBS="-L${trydir} -ltcl ${tcltk_saved_LIBS}"
          AC_LINK_IFELSE(
            [AC_LANG_PROGRAM([#include <tcl.h>
              ], [
                int ac;
                char *av[1];
                Tcl_Main(ac, av, Tcl_AppInit);
                return 0;
              ])], [
                AC_MSG_RESULT([${trydir}])
                TCL_LIBS="-L${trydir} -ltcl"
                break
          ])
        ])
      done

      AS_IF([test ${TCL_LIBS:-unset} = "unset"], [
        AC_MSG_RESULT([not found])
        AC_MSG_ERROR([Could not find TCL library])
      ], [
        # Temporarily update LIBS for the benefit of the Tk detection that follows
        LIBS="${TCL_LIBS} ${X_PRE_LIBS} ${X_LIBS} ${X_EXTRA_LIBS} ${tcltk_saved_LIBS}"
      ])
  
      AC_LANG_POP([C])
    ])
  ], [
    # The user explicitly specified TCL_LIBS and / or TCL_CPPFLAGS; make sure they work
    AC_MSG_CHECKING([Verifying user-specified TCL flags])
    CPPFLAGS="$CPPFLAGS ${TCL_CPPFLAGS}"
    LIBS="${TCL_LIBS} ${tcltk_saved_LIBS}"
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM([#include <tcl.h>
        ], [
          int ac;
          char *av[1];
          Tcl_Main(ac, av, Tcl_AppInit);
          return 0;
        ])],
      [AC_MSG_RESULT([succeeded])],
      [AC_MSG_RESULT([failed])
        AC_MSG_ERROR([User-provided TCL compilation and linking flags did not work])
    ])
  ])


  # Defer to user-specified TK_LIBS and/or TK_CPPFLAGS if they are provided
  AS_IF([test "x${TK_LIBS}${TK_CPPFLAGS}" = x], [
    AS_IF([test "x$tk_base" = x], [
      # --with-tk was not given, so just check the default search path
      AC_CHECK_LIB([tk], [Tcl_Main], [TK_LIBS="-ltk"],
        [AC_MSG_ERROR([Tk library is not found.  Try specifing its location via --with-tk])])
    ], [
      # --with-tk was given, so attempt to find the headers and libs under the specified base
      AS_IF([test -r "${tk_base}/include/tk.h"], [
        TK_CPPFLAGS="-I${tk_base}/include"
        CPPFLAGS="${TK_CPPFLAGS} ${CPPFLAGS}"
      ], [AC_MSG_WARN([${tk_base}/include/tk.h does not exist or is not readable.])])

      AC_MSG_CHECKING([for Tk libs in user-specified base directory ${tk_base}])

      AC_LANG_PUSH([C])
      AS_UNSET([TK_LIBS])
      # check lib64/ first to find 64-bit libs on systems that put 32-bit libs in lib/
      saved_base_LIBS=${LIBS}
      for libdir in lib64 lib lib32; do
        trydir="${tk_base}/${libdir}"
        AS_IF([test -d "${trydir}"], [
          LIBS="-L${trydir} -ltk ${saved_base_LIBS}"
          AC_LINK_IFELSE([
            AC_LANG_PROGRAM(
            [#include <tk.h>
            ], [
              Tcl_Interp *interp = NULL;
              Tk_Window parent;
              char name[] = "";
              char topLevScreen[] = "";
              Tk_CreateWindow(interp, parent, name, topLevScreen);
              return 0;
            ])
          ], [
            AC_MSG_RESULT([${trydir}])
            TK_LIBS="-L${trydir} -ltk"
            break
          ])
        ])
      done

      AS_IF([test ${TK_LIBS:-unset} = "unset"], [
        AC_MSG_RESULT([not found])
        AC_MSG_ERROR([Could not find Tk library])
      ])
  
      AC_LANG_POP([C])
    ])
  ], [
    # The user explicitly specified TK_LIBS and / or TK_CPPFLAGS; make sure they work
    AC_MSG_CHECKING([Verifying user-specified Tk flags])
    CPPFLAGS="$CPPFLAGS ${TK_CPPFLAGS}"
    LIBS="${TK_LIBS} ${LIBS}"
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM([#include <tk.h>
        ], [
          Tcl_Interp *interp = NULL;
          Tk_Window parent;
          char name[] = "";
          char topLevScreen[] = "";
          Tk_CreateWindow(interp, parent, name, topLevScreen);
          return 0;
        ])],
      [AC_MSG_RESULT([succeeded])],
      [AC_MSG_RESULT([failed])
        AC_MSG_ERROR([User-provided Tk compilation and linking flags did not work])
    ])
  ])

  # restore the original CPPFLAGS and LIBS
  CPPFLAGS=${tcltk_saved_CPPFLAGS}
  LIBS=${tcltk_saved_LIBS}
])

